/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
import flash.net.URLVariables;

import mx.collections.ArrayCollection;
import mx.controls.Alert;
import mx.managers.PopUpManager;

import org.seasar.tugboat.gtd.auth.Authorize;
import org.seasar.tugboat.gtd.commons.Constants;
import org.seasar.tugboat.gtd.dto.ActivityDto;
import org.seasar.tugboat.gtd.dto.CustomComboBoxItemDto;
import org.seasar.tugboat.gtd.webservices.RESTWebService;

private static var _MODE_NEW:int  = 0;
private static var _MODE_EDIT:int = 1;
private var _mode:int;
private var _activity:ActivityDto;
private var _closure:Function;

/**
 * initialize.
 * 
 * @param dto - activity dto.
 * @param closure
 */
public function init(dto:ActivityDto, closure:Function):void {
  this._mode = dto == null ? _MODE_NEW : _MODE_EDIT;
  this._activity = dto;
  this._closure = closure;

  var colors:Array = [];
  for (var i:int = 0; i < Constants.ACTIVITIY_COLORS.length; i++) {
    colors.push(
      new CustomComboBoxItemDto(
        Constants.ACTIVITIY_COLORS[i].id,
        Constants.ACTIVITIY_COLORS[i].label,
        Constants.ACTIVITIY_COLORS[i].icon
      )
    );
  }
  
  if (this._mode == _MODE_NEW) {
    this._activity = new ActivityDto(-1, "", -1, "");
    this.title = "New Activity";
    this.colors.dataProvider = new ArrayCollection(colors);
    this.colors.selectedIndex = -1;
    
    this.activity_name.text = "";
    this.description.text = "";
    
  } else {
    this.title = "Activity Detail";
    var selected_color_id:int = Constants.get_activity_color(dto.color).id;
    
    this.colors.setItems(colors, selected_color_id);
    this.activity_name.text = dto.name;
    this.description.text = dto.description;
  }
}

/**
 * close window.
 * 
 */
private function _close_window():void {
  PopUpManager.removePopUp(this);
}

/**
 * post.
 * 
 */
private function _post():void {
  this._activity.name = this.activity_name.text;
  this._activity.description = this.description.text;
  this._activity.color = 
    this.colors.selectedIndex == -1
    ? -1
    : this.colors.selectedItem.id;
  
  var messages:Array = [];
  if (this._activity.color == -1) messages.push("Color is required.");
  if (this._activity.name == "")  messages.push("Name is required.");
  if (messages != null && messages.length > 0) {
    var message:String = "";
    for (var i:int = 0; i < messages.length; i++) {
      message += messages[i];
      message += "\n";
    }
    Alert.show(message,"error message");
    return;
  }
  
  if (this._mode == _MODE_NEW) this._new_action();
  else                         this._edit_action();
}

/**
 * new activity action.
 * 
 */
private function _new_action():void {
  var rest:RESTWebService = new RESTWebService();
  var variables:URLVariables = new URLVariables();
  variables.color = this._activity.color;
  variables.name = this._activity.name;
  variables.description = this._activity.description;
  
  rest.do_post(
    Authorize.root_url + Constants.URI_ACTIVITIES,
    variables,
    _after_posted_on_complete,
    _after_posted_on_failure
  );
}

/**
 * edit activity action.
 * 
 */
private function _edit_action():void {
  var rest:RESTWebService = new RESTWebService();
  var variables:URLVariables = new URLVariables();
  variables.color = this._activity.color;
  variables.name = this._activity.name;
  variables.description = this._activity.description;
  
  rest.do_update(
    Authorize.root_url + Constants.URI_ACTIVITIES + "/" + this._activity.id,
    variables,
    _after_posted_on_complete,
    _after_posted_on_failure
  );
}

/**
 * after posted on complete.
 * 
 * @param e - event
 */
private function _after_posted_on_complete(e:Event):void {
  this._closure();
  this._close_window();
}

/**
 * after posted on failure.
 * 
 */
private function _after_posted_on_failure():void {
  this._close_window();
}