/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
import com.adobe.serialization.json.JSON;

import component.ActivityDetailPopUpWindow;
import component.NewActivityMemberPopUpWindow;

import flash.events.Event;

import mx.collections.ArrayCollection;
import mx.events.ItemClickEvent;
import mx.managers.PopUpManager;

import org.seasar.tugboat.gtd.commons.Constants;
import org.seasar.tugboat.gtd.dto.ActivityDto;
import org.seasar.tugboat.gtd.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.webservices.RESTWebService;

private var _activities:Array;  // for combobox list-item. 

/**
 * initialize.
 * 
 */
private function _init():void {
  this._load_activities();
}

/**
 * load activities.
 * 
 */
private function _load_activities():void {
  var rest:RESTWebService = new RESTWebService();
  rest.do_get(
    Constants.URI_ACTIVITIES, _load_activities_onComplete, _load_onFailure);
}

/**
 * load activities on complete.
 * 
 * @param e - event
 */
private function _load_activities_onComplete(e:Event):void {
  var loader:URLLoader = URLLoader(e.target);
  var response_text:String = loader.data;
  var array:Array = 
    (response_text == null || response_text == "null" || response_text == "")
    ? null
    : JSON.decode(response_text) as Array;
  
  this._activities = [];
  for (var i:int = 0; i < array.length; i++) {
    var activity:ActivityDto =
      new ActivityDto(
        array[i].id,
        array[i].value,
        new Number("0x" + array[i].color),
        array[i].description
      );
    this._activities.push(activity);
  }
  this.activity_folders.dataProvider = new ArrayCollection(this._activities);
  this.activity_folders.selectedIndex = 0;
  this._load_activity_members(this._activities[0].id);
}

/**
 * load activity members by activity id.
 * 
 * @param activity id
 */
private function _load_activity_members(activity_id:int):void {
  var rest:RESTWebService = new RESTWebService();
  rest.do_get(
    Constants.URI_ACTIVITY_MEMBERS + "/activityId/" + activity_id,
    _load_activity_members_onComplete,
    _load_onFailure
  );
}

/**
 * load activity members on complete.
 * 
 * @param e - event
 * 
 */
private function _load_activity_members_onComplete(e:Event):void {
  var loader:URLLoader = URLLoader(e.target);
  var response_text:String = loader.data;
  var members:Array = 
    (response_text == null || response_text == "null" || response_text == "")
    ? null
    : JSON.decode(response_text) as Array;
  
  var list:Array = [];
  if (members == null) {
    this.list_view.dataProvider = new ArrayCollection(list);
    return;
  }
  for (var i:int = 0; i < members.length; i++) {
    var dto:ActivityMemberDto =
      new ActivityMemberDto(
        members[i].id,
        members[i].firstName,
        members[i].lastName,
        members[i].roleId,
        members[i].join
      );
    list.push(dto);
  }
  this.list_view.dataProvider = new ArrayCollection(list);
}

/**
 * load on failure.
 * 
 */
private function _load_onFailure(e:Object):void {
  trace('::: error :::');
  trace(e);
}

/**
 * activity on change (selected).
 * 
 */
private function _activity_onChange():void {
  var selected_item:Object = this.activity_folders.selectedItem;
  this._load_activity_members(selected_item.id);
}

/**
 * move to main.
 * 
 */
private function _move_to_main():void {
  this.dispatchEvent(new Event("move_to_main",false,false));
}

/**
 * show detail sub window.
 * 
 */
private function _show_detail_sub_window():void {
  var detail_window:ActivityDetailPopUpWindow = 
    ActivityDetailPopUpWindow(
      PopUpManager.createPopUp(this, ActivityDetailPopUpWindow, true)
    );
    
    detail_window.init(
      this.activity_folders.selectedItem as ActivityDto,
      function():void {
        _load_activities();
      }
    );
    detail_window.x = (this.width  - detail_window.width)  / 2;
    detail_window.y = (this.height - detail_window.height) / 2;
}

/**
 * menu event handler.
 * 
 * @param event
 */
private function _menu_event_handler(event:ItemClickEvent):void {
  switch (event.index) {
    //
    // new activity.
    //
    case 0:
      var detail_window:ActivityDetailPopUpWindow = 
        ActivityDetailPopUpWindow(
          PopUpManager.createPopUp(this, ActivityDetailPopUpWindow, true)
        );
      
      detail_window.init(
        null,
        function():void {
          _load_activities();
        }
      );
      detail_window.x = (this.width  - detail_window.width)  / 2;
      detail_window.y = (this.height - detail_window.height) / 2;
      break;

    //
    // new member.
    //
    case 1:
      var activity:ActivityDto = 
        this._activities[this.activity_folders.selectedIndex];
      
      var member_window:NewActivityMemberPopUpWindow = 
        NewActivityMemberPopUpWindow(
          PopUpManager.createPopUp(this, NewActivityMemberPopUpWindow, true)
        );
      
      member_window.init(
        activity,
        null,
        function():void {
          _load_activity_members(activity.id);
        }
      );
      member_window.x = (this.width  - member_window.width)  / 2;
      member_window.y = (this.height - member_window.height) / 2;
      break;
  }
}

/**
 * item on double click.
 * 
 */
private function _item_on_dblclick():void {
  var list_sel:Object = list_view.selectedItem;
  var activity:ActivityDto = 
    this._activities[this.activity_folders.selectedIndex];

  var member_window:NewActivityMemberPopUpWindow = 
    NewActivityMemberPopUpWindow(
      PopUpManager.createPopUp(this, NewActivityMemberPopUpWindow, true)
    );
  member_window.init(
    activity,
    list_view.selectedItem as ActivityMemberDto,
    function():void {
      _load_activity_members(activity.id);
    }
  );
  member_window.x = (this.width  - member_window.width)  / 2;
  member_window.y = (this.height - member_window.height) / 2;
}