/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
import com.adobe.serialization.json.JSON;

import flash.events.Event;
import flash.net.URLLoader;

import mx.collections.ArrayCollection;

import org.seasar.tugboat.gtd.auth.Authorize;
import org.seasar.tugboat.gtd.commons.Constants;
import org.seasar.tugboat.gtd.dto.BrowseClassificationDto;
import org.seasar.tugboat.gtd.dto.BrowsePersonInChargeDto;
import org.seasar.tugboat.gtd.dto.BrowserActivityDto;
import org.seasar.tugboat.gtd.webservices.RESTWebService;
  
private var _selected_folder_id:Number = 0;

/**
 * setup activities.
 * @param array
 */
public function setup_activities(array:Array):void {
  var activities:Array = [
    new BrowserActivityDto(0, "All", 0x0, "images/default/icon/space.gif", true)
  ];
  for (var i:Number = 0; i < array.length; i++) {
    var activity:Object =
      Constants.get_activity_color(
        new Number("0x"+array[i].color));
    
    activities.push(
      new BrowserActivityDto(
        array[i].id, array[i].value, new Number("0x"+array[i].color), activity.icon, false)
    );
  }
  this.activities.dataProvider = new ArrayCollection(activities);
}

/**
 * setup person in charge.
 * 
 */
public function setup_person_in_charge(array:Array):void {
  var person_in_charges:Array = [
    new BrowsePersonInChargeDto(0, "All", "", 0, false)
  ];
  
  for (var i:Number = 0; i < array.length; i++) {
    if (Authorize.user_id == array[i].userId) {
      person_in_charges.push(
        new BrowsePersonInChargeDto(
          array[i].userId,
          "[Me]",
          "",
          array[i].roleId,
          true
        )
      );
      break;
    }
  }
  for (i = 0; i < array.length; i++) {
    if (Authorize.user_id != array[i].userId) {
      person_in_charges.push(
        new BrowsePersonInChargeDto(
          array[i].userId,
          array[i].lastName + ", ",
          array[i].firstName,
          array[i].roleId,
          false
        )
      );
    }
  }
  this.person_in_charges.dataProvider =
    new ArrayCollection(person_in_charges);
}

/**
 * selected class folder.
 * @param id
 */
public function selected_class_folder(id:Number):void {
  this._selected_folder_id = id;
  this.dispatchEvent(new Event("selected"));
}

/**
 * classes on click event handler.
 * @param event
 */
public function classes_onclick(event:Event):void {
  this._checked_browser_item(
    classes.selectedItem, classes.dataProvider as ArrayCollection);
  
  this.dispatchEvent(new Event("selected"));
}

/**
 * activities on click event handler.
 * @param event
 */
public function activities_onclick(event:Event):void {
  var _activities:ArrayCollection = activities.dataProvider as ArrayCollection;
  this._checked_browser_item(activities.selectedItem, _activities);
    
  var activity_ids:Array = [];
  for (var i:Number = 0; i < _activities.length; i++) {
    if (_activities[i].selected) {
      activity_ids.push(_activities[i].id);
    }
  }
  this._find_person_in_charges_by_activityIds(activity_ids);
  this.dispatchEvent(new Event("selected"));
}

/**
 * person in charges on click event handler.
 * @param event
 */
public function person_in_charges_onclick(event:Event):void {
  this._checked_browser_item(
    person_in_charges.selectedItem,
    person_in_charges.dataProvider as ArrayCollection);
    
  this.dispatchEvent(new Event("selected"));
}

/**
 * reset browser.
 */
public function reset_browser():void {
  var classes:ArrayCollection = classes.dataProvider as ArrayCollection;
  var activities:ArrayCollection = activities.dataProvider as ArrayCollection;
  var person_in_charges:ArrayCollection = person_in_charges.dataProvider as ArrayCollection;
  
  for (var i:Number = 0; i < classes.length; i++) {      
    classes[i].selected = (i == 0);
  }
  for (i = 0; i < activities.length; i++) { 
    activities[i].selected = (activities[i].id == 0);
  }
  for (i = 0; i < person_in_charges.length; i++) {      
    person_in_charges[i].selected =
      (Authorize.user_id == person_in_charges[i].id);
  }
  this.dispatchEvent(new Event("selected"));
}

/**
 * load person in charge.
 * 
 */
private function _load_person_in_charge():void {
  var rest:RESTWebService = new RESTWebService();
  rest.do_get(
    Constants.URI_ACTIVITY_MEMBERS + "/action/findByBelongActivities",
    _load_person_in_charge_onConnected,
    _load_onFailure
  );
}

/**
 * find person in charges by activity ids. 
 * 
 * @param ids - activity ids
 */
private function _find_person_in_charges_by_activityIds(ids:Array):void {
  var ids_string:String = "";
  for (var i:Number = 0; i < ids.length; i++) {
    var id:Number = ids[i];
    ids_string += (i == ids.length - 1 ? new String(id) : new String(id) + ",");
  }
  
  if (ids_string == "0") {
    return this._load_person_in_charge();
  }
  var rest:RESTWebService = new RESTWebService();
  rest.do_get(
    Constants.URI_ACTIVITY_MEMBERS + "/activityIds/" + ids_string,
    _load_person_in_charge_onConnected,
    _load_onFailure
  );
}

/**
 * load person in charge on connected.
 * @param e - event 
 * 
 */
private function _load_person_in_charge_onConnected(e:Event):void {
  var loader:URLLoader = URLLoader(e.target);
  var response_text:String = loader.data;
  if (response_text == null || response_text == "null" || response_text == ""){
    return;
  }
  var array:Array = (JSON.decode(response_text) as Array);
  this.setup_person_in_charge(array);
}

/**
 * checked browser item. 
 * 
 * @param selected_item
 * @param data_provider
 */
private function _checked_browser_item(selected_item:Object, data_provider:ArrayCollection):void {
  if (selected_item == null) return;
  if ((selected_item is BrowseClassificationDto && selected_item.enable)
  || !(selected_item is BrowseClassificationDto)) {
    selected_item.selected = !selected_item.selected;
  }
  
  if (selected_item.id == 0) {
    for (var i:Number = 0; i < data_provider.length; i++) {
      if (data_provider[i].id == 0) continue;
      data_provider[i].selected = false;
    }
  } else if (selected_item.id > 0) {
    data_provider[0].selected = false;
  } 
  
  var is_exists_selected:Boolean = false;
  for (i = 0; i < data_provider.length; i++) {
    if (data_provider[i].selected) {
      is_exists_selected = true;
      break;
    }
  }
  if (!is_exists_selected) {
    data_provider[0].selected = true;
  }
}

/**
 * load on failure.
 * 
 */
private function _load_onFailure():void {
  trace('::::: error#_load_onFailure :::::');
}
