/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
import flash.events.Event;

import mx.containers.HBox;
import mx.events.DragEvent;
import mx.managers.DragManager;

import org.seasar.tugboat.gtd.commons.Constants;
import org.seasar.tugboat.gtd.dto.TaskFolderDto;

private var _height_gtd_section:int = 0;
private var _selected:HBox = null;
private var _is_visibled_gtd_section:Boolean = true;
private var _dropped_folder_id:int = 0;

[Bindable]
public var task_folder_dto:TaskFolderDto;

/**
 * initialize.
 */
private function _init():void {
  _selected = this.folder_inbox;
  _height_gtd_section = this.gtd_folder.height;
}

/**
 * selected folder on click handler
 * @param id - folder id.
 */
private function _selected_folder_onclick(selected:HBox):void {
  if (this.get_selected_folderId() == selected.name) {
    return;
  }
  this.folder_onclick(selected);
  this.dispatchEvent(new Event("selected"));
}

/**
 * get selected folder id.
 * @return - folder id.
 */
public function get_selected_folderId():String {
  return this.get_selected_id();
}

/**
 * get dropped folder id.
 * @return - foder id.
 */
public function get dropped_folder_id():int {
  return this._dropped_folder_id;
}

/**
 * accept drop.
 * 
 * @param event
 */
private function _drag_enter(event:DragEvent):void {
  var target:* = event.currentTarget;
  mx.managers.DragManager.acceptDragDrop(target);

  this._set_drag_over_style(target.id);
}

private function _drag_over(event:DragEvent):void {
  var target:* = event.currentTarget;
  this._set_drag_over_style(target.id);
}

/**
 * set drag over style.
 * 
 * @param id
 */
private function _set_drag_over_style(id:String):void {
  switch (id) {
    case "folder_inbox":       this._set_folder_style(folder_inbox, true);       break;
    case "folder_today_todo":  this._set_folder_style(folder_today_todo, true);  break;
    case "folder_today_doing": this._set_folder_style(folder_today_doing, true); break;
    case "folder_today_done":  this._set_folder_style(folder_today_done, true);  break;
    case "folder_week":        this._set_folder_style(folder_week, true);        break;
    case "folder_month":       this._set_folder_style(folder_month, true);       break;
    case "folder_hold":        this._set_folder_style(folder_hold, true);        break;
    case "folder_someday":     this._set_folder_style(folder_someday, true);     break;
    case "folder_trash":       this._set_folder_style(folder_trash, true);       break;
  }
}

/**
 * set folder style.
 * 
 * @param folder
 * @param is drag enter
 */
private function _set_folder_style(folder:HBox, is_drag_enter:Boolean):void {
  if (is_drag_enter) {
    folder.setStyle("backgroundColor",Constants.COLOR_FOLDER_DRAG_ENTER);
    folder.height = 40;
  } else {
    var background_style:String = 
      folder.id == _selected.id
      ? Constants.COLOR_FOLDER_SELECTED
      : Constants.COLOR_FOLDER_DEFAULT; 
    
    folder.setStyle("backgroundColor", background_style);
    folder.height = 20;
  }
}

/**
 * drag out.
 * 
 * @param event
 */
private function _drag_out(event:DragEvent):void {
  var target:* = event.currentTarget;
  this._set_drag_out_style(target.id);
}

/**
 * drag'n drop.
 */
private function _drag_drop_event_handler(event:DragEvent):void {
  var target:* = event.currentTarget;
  this._set_drag_out_style(target.id);
  this._dropped_folder_id = Constants.get_folder(target.id).id;
  this.dispatchEvent(new Event("dropped"));
}

/**
 * set drag out style.
 * @param id - folder id.
 */
private function _set_drag_out_style(id:String):void {
  switch (id) {
    case "folder_inbox":       this._set_folder_style(folder_inbox,false);       break;
    case "folder_today_todo":  this._set_folder_style(folder_today_todo,false);  break;
    case "folder_today_doing": this._set_folder_style(folder_today_doing,false); break;
    case "folder_today_done":  this._set_folder_style(folder_today_done,false);  break;
    case "folder_week":        this._set_folder_style(folder_week,false);        break;
    case "folder_month":       this._set_folder_style(folder_month,false);       break;
    case "folder_hold":        this._set_folder_style(folder_hold,false);        break;
    case "folder_someday":     this._set_folder_style(folder_someday,false);     break;
    case "folder_trash":       this._set_folder_style(folder_trash,false);       break;
  }
}

/**
 * get selected id. 
 * @return
 */
public function get_selected_id():String {
  return this._selected.name;
}

/**
 * selected folder.
 * 
 * @param folder id
 */
public function selected_folder(folder_id:String):void {
  var folder:HBox;
  switch (folder_id) {
    case "folder_inbox":       folder = folder_inbox;       break;
    case "folder_today_todo":  folder = folder_today_todo;  break;
    case "folder_today_doing": folder = folder_today_doing; break;
    case "folder_today_done":  folder = folder_today_done;  break;
    case "folder_week":        folder = folder_week;        break;
    case "folder_month":       folder = folder_month;       break;
    case "folder_hold":        folder = folder_hold;        break;
    case "folder_someday":     folder = folder_someday;     break;
    case "folder_trash":       folder = folder_trash;       break;
  }
  this.folder_onclick(folder);
}

/**
 * toggle gtd section.
 *
 */  
private function toggle_gtd_section():void {
  this.gtd_folder.height = 
    _is_visibled_gtd_section ? 0 : _height_gtd_section;

  this.gtd_folder.visible = !_is_visibled_gtd_section;
  _is_visibled_gtd_section = !_is_visibled_gtd_section;
}

/**
 * folder on click.
 *
 * @param hbox
 */
private function folder_onclick(hbox:HBox):void {
  _selected.setStyle(
    "backgroundColor", Constants.COLOR_FOLDER_DEFAULT);

  var id:Number = 0;

  if (hbox.id.indexOf("folder_today") > -1) {
    this.folder_today.setStyle(
      "backgroundColor", Constants.COLOR_FOLDER_SELECTED);
    _selected = this.folder_today;
  } else {
    hbox.setStyle(
      "backgroundColor", Constants.COLOR_FOLDER_SELECTED);
    _selected = hbox;
  }
}

/**
 * folder on mouse over.
 *
 * @param hbox
 */
private function folder_onmouseover(hbox:HBox):void {
  if (hbox.id == _selected.id) return;
  hbox.setStyle("backgroundColor", Constants.COLOR_FOLDER_HOVER);
}
/**
 * on mouse out.
 *
 * @param hbox
 */
private function folder_onmouseout(hbox:HBox):void {
  if (hbox.id == _selected.id) return;
  hbox.setStyle("backgroundColor", Constants.COLOR_FOLDER_DEFAULT);
}