/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
import com.adobe.serialization.json.JSON;

import flash.events.Event;
import flash.net.URLLoader;
import flash.net.URLVariables;

import mx.collections.ArrayCollection;
import mx.controls.Alert;
import mx.managers.PopUpManager;

import org.seasar.tugboat.gtd.auth.Authorize;
import org.seasar.tugboat.gtd.commons.Constants;
import org.seasar.tugboat.gtd.dto.ActivityDto;
import org.seasar.tugboat.gtd.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.dto.CustomComboBoxItemDto;
import org.seasar.tugboat.gtd.webservices.RESTWebService;

private static var _MODE_NEW:int  = 0;
private static var _MODE_EDIT:int = 1;
private var _mode:int;
private var _activity:ActivityDto;
private var _member:ActivityMemberDto;
private var _closure:Function;
private var _member_input_value:String;

/**
 * initialize.
 * 
 * @param activity
 * @param member - activity member dto.
 * @param closure
 */
public function init(activity:ActivityDto, member:ActivityMemberDto, closure:Function):void {
  this._mode = member == null ? _MODE_NEW : _MODE_EDIT;
  this._activity = activity;
  this._member = member;
  this._closure = closure;
  
  this.activity.text = this._activity.name;
  
  var roles:Array = [];
  for (var i:int = 0; i < Constants.USER_ROLES.length; i++) {
    roles.push(
      new CustomComboBoxItemDto(
        Constants.USER_ROLES[i].id,
        Constants.USER_ROLES[i].name,
        Constants.USER_ROLES[i].icon
      )
    );
  }
  
  this._load_members();
  
  if (this._mode == _MODE_NEW) {
    this.new_member_name_box.height = 28;
    this.new_member_name_box.visible = true;
    this.edit_member_name_box.height = 0;
    this.edit_member_name_box.visible = false;
    
    this.title = "New Member";
    
    this.roles.dataProvider = new ArrayCollection(roles);
    this.roles.selectedIndex = -1;
    
    this.new_member_name.text = "";
    
  } else {
    this.new_member_name_box.height = 0;
    this.new_member_name_box.visible = false;
    this.edit_member_name_box.height = 28;
    this.edit_member_name_box.visible = true;
    
    this.title = "Edit Member";
    
    var selected_role_id:int = Constants.get_user_role(member.role_id).id;
    this.roles.setItems(roles, selected_role_id);
    
    this.edit_member_name.text = member.last_name + "," + member.first_name;
  }
}

/**
 * post action.
 * 
 */
private function _post():void {
  var messages:Array = [];
  if (this.roles.selectedIndex == -1) messages.push("Role is required.");
  if (this._mode == _MODE_NEW
   && this.new_member_name.selectedIndex == -1)  messages.push("Member name is required.");
  
  if (messages != null && messages.length > 0) {
    var message:String = "";
    for (var i:int = 0; i < messages.length; i++) {
      message += messages[i];
      message += "\n";
    }
    Alert.show(message,"error message");
    return;
  }
  if (this._mode == _MODE_NEW) this._new_member_action();
  else                         this._update_member_action();
}

/**
 * new member action.
 * 
 */
private function _new_member_action():void {
  var activity_id:int = this._activity.id;
  var user_id:int = new_member_name.selectedItem.id;
  var rest:RESTWebService = new RESTWebService();
  rest.do_get(
    Constants.URI_ACTIVITY_MEMBERS + "/activityId/" + activity_id + "/userId/" + user_id,
    _create_member,
    _on_failure
  );
}

/**
 * create member.
 * 
 * @param e - event.
 */
private function _create_member(e:Event):void {
  var loader:URLLoader = URLLoader(e.target);
  var response_text:String = loader.data;
  var array:Array = 
    (response_text == null || response_text == "null" || response_text == "")
    ? null
    : JSON.decode("[" + response_text + "]") as Array;
  
  if (array != null) {
    Alert.show(
      "'"
    + new_member_name.selectedItem.label 
    + "' is already exists","error");
    return;
  }
  
  var rest:RESTWebService = new RESTWebService();
  var variables:URLVariables = new URLVariables();
  variables.activityId = this._activity.id;
  variables.roleId = roles.selectedItem.id;
  variables.id = new_member_name.selectedItem.id;
  
  rest.do_post(
    Authorize.root_url + Constants.URI_ACTIVITY_MEMBERS,
    variables,
    _posted_onComplete,
    _on_failure
  );
}


/**
 * update member action.
 * 
 */
private function _update_member_action():void {
  var rest:RESTWebService = new RESTWebService();
  var variables:URLVariables = new URLVariables();
  
  variables.id = this._member.id;
  variables.activityId = this._activity.id;
  variables.roleId = roles.selectedItem.id;
  
  rest.do_update(
    Authorize.root_url + Constants.URI_ACTIVITY_MEMBERS,
    variables,
    _posted_onComplete,
    _on_failure
  );
}

/**
 * post on complete.
 * 
 * @param e - event
 */
private function _posted_onComplete(e:Event):void {
  this._closure();
  PopUpManager.removePopUp(this);
}

/**
 * close window action.
 * 
 */
private function _close_window():void {
  PopUpManager.removePopUp(this);
}

/**
 * load members.
 * 
 */
private function _load_members():void {
  var rest:RESTWebService = new RESTWebService();
  rest.do_get(
    Constants.URI_ACTIVITY_MEMBERS + "/action/findAllMembers",
    _load_members_on_complete,
    _on_failure
  );
}

/**
 * suggest on complete
 * 
 * @param e - event.
 */
private function _load_members_on_complete(e:Event):void {
  var loader:URLLoader = URLLoader(e.target);
  var response_text:String = loader.data;
  if (response_text == null || response_text == "null" || response_text == ""){
    this.new_member_name.dataProvider = new ArrayCollection(new Array());

  } else {
    var array:Array = (JSON.decode(response_text) as Array);
    var members:Array = [];
    
    for each (var m:Object in array) {
      members.push(
        new CustomComboBoxItemDto(
          m.userId,
          m.lastName + ", " + m.firstName,
          "images/default/icon/contact_known.gif"
        )
      );
    }
    this.new_member_name.dataProvider = new ArrayCollection(members);
    this.new_member_name.selectedIndex = -1;
    this.new_member_name.text = this._member_input_value;
  }
}

/**
 * on failure.
 * 
 */
private function _on_failure():void {
  this._close_window();
}