/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
import flash.net.URLVariables;

import mx.collections.ArrayCollection;
import mx.controls.Alert;

import org.seasar.tugboat.gtd.auth.Authorize;
import org.seasar.tugboat.gtd.commons.Constants;
import org.seasar.tugboat.gtd.dto.CustomComboBoxItemDto;
import org.seasar.tugboat.gtd.webservices.RESTWebService;

private var _is_posting:Boolean = false;

/**
 * setup activities.
 * @param array
 */
public function setup_activities(array:Array):void {
  var activities:Array = [];
  for (var i:Number = 0; i < array.length; i++) {
    var activity:Object =
      Constants.get_activity_color(
        new Number("0x"+array[i].color));
    
    activities.push(
      new CustomComboBoxItemDto(array[i].id, array[i].value, activity.icon));
  }
  this.activities.dataProvider = new ArrayCollection(activities);
}

/**
 * is visible option.
 * 
 */
public function is_visible_option():Boolean {
  return this.option_area.visible;
}

/**
 * toggle option area.
 * 
 */
private function _toggle_option_area():void {
  if (this.option_area.visible) {
    this.height = 40;
    this.option_area.height = 0;
    this.option_area.visible = false;
    toggle_arrow.source='images/default/icon/bullet_arrow_right.gif';

  } else {
    this.height = 220;
    this.option_area.height = 180;
    this.option_area.visible = true;
    
    toggle_arrow.source='images/default/icon/bullet_arrow_down.gif';
  }
}

/**
 * clear due.
 * 
 */
private function _clear_due():void {
  this.due.text = "";
}

/**
 * cancel.
 */
private function _cancel():void {
  this.title.text = "";
  this.description.text = "";
  this.due.text = "";
}

/**
 * post.
 * 
 */ 
private function _post():void {
  if (_is_posting) return;
  _is_posting = true;
  
  var activity_id:int = this.activities.getSelectedItemId();
  var title:String = this.title.text;
  var description:String = this.description.text;
  var due:String = this.due.text;
  if (due != "" && due.length == 10) {
    due = due.substr(0,4) + due.substr(5,2) + due.substr(8,2);
  }

  var messages:Array = [];
  if (activity_id == 0)  messages.push("Activity is required.");
  if (title == "")  messages.push("Title is required.");
  if (messages != null && messages.length > 0) {
    var message:String = "";
    for (var i:int = 0; i < messages.length; i++) {
      message += messages[i];
      message += "\n";
    }
    Alert.show(message,"error message");
    _is_posting = false;
    return;
  }
  
  // post.rest;
  var rest:RESTWebService = new RESTWebService();
  var variables:URLVariables = new URLVariables();
  variables.activity = activity_id;
  variables.title = title;
  variables.description = description;
  variables.due = due;
  
  rest.do_post(
    Authorize.root_url + Constants.URI_TASKS,
    variables,
    _after_posted_on_complete,
    _after_posted_on_failure
  );
}

/**
 * after posted on complete.
 * 
 * @param e - event
 */
private function _after_posted_on_complete(e:Event):void {
  _is_posting = false;
  
  this.dispatchEvent(new Event("posted"));
  this._cancel();
}

/**
 * after posted on error.
 * 
 */
private function _after_posted_on_failure():void {
  _is_posting = false;
}
