/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */ 
import mx.collections.ArrayCollection;
import mx.controls.Alert;
import mx.managers.PopUpManager;

import org.seasar.tugboat.gtd.auth.Authorize;
import org.seasar.tugboat.gtd.commons.Constants;
import org.seasar.tugboat.gtd.dto.CustomComboBoxItemDto;
import org.seasar.tugboat.gtd.dto.TaskDto;
import org.seasar.tugboat.gtd.webservices.RESTWebService;

private var _task:TaskDto = null;
private var _is_posting:Boolean = false;
private var _after_posted_proc:Function = null;

/**
 * set task
 * @param dto
 */
public function setTask(task:TaskDto):void {
  this._task = task;
  
  this.title_text.text = this._task.title;
  this.description.text = this._task.description;
  this.due.text = this._task.due;
}

/**
 * setup activities.
 * 
 * @param array
 * @param activity id
 */
public function setup_activities(array:Array, activity_id:int):void {
  var activities:Array = [];
  var selected_index:int = 0;
  
  for (var i:Number = 0; i < array.length; i++) {
    var activity:Object =
      Constants.get_activity_color(
        new Number("0x"+array[i].color));
    
    if (array[i].id == activity_id) {
      selected_index = i;
    }
    activities.push(
      new CustomComboBoxItemDto(array[i].id, array[i].value, activity.icon));
  }
  this.activities.dataProvider = new ArrayCollection(activities);
  this.activities.selectedIndex = selected_index;
}

/**
 * setup folders.
 * 
 * @param array
 * @param class_id
 * @param status_id
 */
public function setup_folders(array:Array, class_id:int, status_id:int):void {
  var folders:Array = [];
  var selected_index:int = -1;
  var index:int = 0;
  
  for (var i:int = 0; i < array.length; i++) {
    if (! array[i].is_selectable) continue;

    if (array[i].classId   == class_id
     && array[i].status_id == status_id) {
      selected_index = index;
    }
    folders.push(
      new CustomComboBoxItemDto(
        array[i].id, array[i].value, array[i].icon)
    );
    index++;
  }
  this.folders.dataProvider = new ArrayCollection(folders);
  this.folders.selectedIndex = selected_index;
}

/**
 * after posted process.
 * 
 * @param closure - function.
 */
public function after_posted_proc(closure:Function):void {
  this._after_posted_proc = closure;
}

/**
 * close window.
 * 
 */
private function _close_window():void {
  PopUpManager.removePopUp(this);
}

/**
 * post action.
 * 
 */
private function _post():void {
  if (_is_posting) return;
  _is_posting = true;
  
  var folder_id:int = this.folders.getSelectedItemId();
  var activity_id:int = this.activities.getSelectedItemId();
  var title:String = this.title_text.text;
  var description:String = this.description.text;
  var due:String = this.due.text;
  if (due != "" && due.length == 10) {
    due = due.substr(0,4) + due.substr(5,2) + due.substr(8,2);
  }
  
  var messages:Array = [];
  if (folder_id == 0) messages.push("Folder is required.");
  if (activity_id == 0)  messages.push("Activity is required.");
  if (title == "")  messages.push("Title is required.");
  if (messages != null && messages.length > 0) {
    var message:String = "";
    for (var i:int = 0; i < messages.length; i++) {
      message += messages[i];
      message += "\n";
    }
    Alert.show(message,"error message");
    _is_posting = false;
    return;
  }
  
  // posted.action.
  var status_id:int = 
    folder_id == 11
 || folder_id == 2
 || folder_id == 3
 || folder_id == 4
 || folder_id == 5
 || folder_id == 9
    ? 1
    : folder_id == 12
      ? 2
      : 3;
    
  var class_id:int =
      folder_id == 11 || folder_id == 12 || folder_id == 13 ? 1
    : folder_id == 2 ? 2 
    : folder_id == 3 ? 3
    : folder_id == 4 ? 4
    : folder_id == 4 || folder_id == 43 ? 4
    : folder_id == 5 ? 5
    : folder_id == 9 ? 9
                     : 99;
  
  var rest:RESTWebService = new RESTWebService();
  var variables:URLVariables = new URLVariables();
  variables.activity = activity_id;
  variables.status = status_id;
  variables.classification = class_id;
  variables.title = title;
  variables.description = description;
  variables.due = due;
  
  rest.do_update(
    Authorize.root_url + Constants.URI_TASKS + "/" + this._task.id,
    variables,
    _after_posted_on_complete,
    _after_posted_on_failure
  );
}

/**
 * after posted action on complete.
 * 
 * @param e - event
 */
private function _after_posted_on_complete(e:Event):void {
  this._is_posting = false;
  this._after_posted_proc();
  this._close_window();
}

/**
 * after posted action on failure.
 * 
 */
private function _after_posted_on_failure():void {
  this._is_posting = false;
}

/**
 * calendar clear.
 * 
 */
private function _calendar_clear():void {
  this.due.text = "";
}