/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
import com.adobe.serialization.json.JSON;
import com.adobe.utils.StringUtil;

import component.TaskEditPopUpWindow;

import flash.events.Event;
import flash.net.URLLoader;

import mx.collections.ArrayCollection;
import mx.controls.Alert;
import mx.events.CloseEvent;
import mx.events.ItemClickEvent;
import mx.managers.PopUpManager;

import org.seasar.tugboat.gtd.auth.Authorize;
import org.seasar.tugboat.gtd.commons.Constants;
import org.seasar.tugboat.gtd.dto.BrowseClassificationDto;
import org.seasar.tugboat.gtd.dto.TaskDto;
import org.seasar.tugboat.gtd.dto.TaskFolderDto;
import org.seasar.tugboat.gtd.dto.TaskQueryConditionDto;
import org.seasar.tugboat.gtd.webservices.RESTWebService;

private var _offset:Number = 0;
private var _limit:Number = Constants.LINES_PER_PAGE;
private var _selected_folder_id:Number;

private var _activities:Array;  // for combobox list-item.

[Bindable]
public var dataProvider:ArrayCollection;

public var task_folder_dto:TaskFolderDto = new TaskFolderDto();

/**
 * initialize.
 * 
 */
public function init():void {
  this._gtd_folder_click_event_handler(Constants.FOLDER_INBOX);
  this._load_activities();
  this._load_person_in_charge();
}

/**
 * menu1 event handler.
 * 
 * @param event
 */
private function menu1_event_handler(event:ItemClickEvent):void {
  switch (event.index) {
    case 0:
      this._gtd_folder_click_event_handler(Constants.FOLDER_INBOX);
      this.folder_pane.selected_folder(Constants.FOLDER_INBOX);
      break;
    case 1:
      Alert.show(
        "May I empty a trash box ?",
        "Confirm",
        Alert.OK|Alert.CANCEL,
        this,
        _empty_trash_conf_on_close
      );
      break;
  }
}

/**
 * empty trash confirm dialog on close.
 * 
 * @param e
 */
private function _empty_trash_conf_on_close(e:CloseEvent):void {
  if (e.detail == Alert.OK) {
    this._empty_trash();
  }
}

/**
 * menu2 event handler.
 * 
 * @param event
 */
private function menu2_event_handler():void {
  is_visibled = !is_visibled;
  browse_pane.visible = is_visibled;
  browse_pane.height = is_visibled ? 141 : 0;

  if (! is_visibled) {
    browse_pane.reset_browser();
  }
}
private var is_visibled:Boolean = false;

/**
 * gtd folder click event handler.
 * 
 * @param selected_folder_id
 */
private function _gtd_folder_click_event_handler(selected_folder_id:String):void {
  var folder:Object = Constants.get_folder(selected_folder_id);
  
  this.selected_folder_id = folder.classId;
  this._selected_gtd_folder(folder.classId);
  
  this.navi_bar_pane.selected_value =
    Constants.get_classification(folder.classId).value;

  this.folder_pane.task_folder_dto = this.task_folder_dto;

  this.new_task_pane.visible =
    selected_folder_id == 'folder_inbox';
  this.new_task_pane.height = 
    selected_folder_id == 'folder_inbox' && this.new_task_pane.is_visible_option()
    ? 220
    : selected_folder_id == 'folder_inbox' && !this.new_task_pane.is_visible_option()
      ? 40
      : 0;
}

/**
 * gtd folder dropped event handler.
 * 
 * @param folder id
 */
private function _gtd_folder_dropped_event_handler(folder_id:int):void {
  var list_sel:Object = list_view.selectedItem;
  var block_sel:Object = block_view.selectedItem;
  
  if (list_sel == null && block_sel == null) return;
  
  var task_id:int = block_view.visible ? block_sel.id : list_sel.id;
  var status_id:int = 
    folder_id == 11
 || folder_id == 2
 || folder_id == 3
 || folder_id == 4
 || folder_id == 5
 || folder_id == 9
 || folder_id == 99
    ? 1
    : folder_id == 12
      ? 2
      : 3;
    
  var class_id:int =
      folder_id == 11 || folder_id == 12 || folder_id == 13 ? 1
    : folder_id == 2 ? 2 
    : folder_id == 3 ? 3
    : folder_id == 4 ? 4
    : folder_id == 5 ? 5
    : folder_id == 9 ? 9
                     : 99;
  
  var rest:RESTWebService = new RESTWebService();
  var variables:URLVariables = new URLVariables();
  variables.status = status_id;
  variables.classification = class_id;
  
  rest.do_update(
    Authorize.root_url + Constants.URI_TASKS + "/" + task_id,
    variables,
    _after_dropped_on_complete,
    _load_onFailure
  );
}

/**
 * after dropped on complete.
 * 
 * @param e - event
 */
private function _after_dropped_on_complete(e:Event):void {
  this._do_query();
}

/**
 * browser item selected click event handler.
 * 
 */
private function _browser_item_selected_click_event_handler():void {
  this._do_query();
  this.folder_pane.task_folder_dto = this.task_folder_dto;
}

/**
 * set offset.
 * 
 * @param offset
 */
private function set offset(offset:Number):void {
  this._offset = offset;
}

/**
 * set limit.
 * 
 * @param limit 
 */
private function set limit(limit:Number):void {
  this._limit = limit;
}

/**
 * set selected folder id.
 * 
 * @param selected_folder_id
 */
private function set selected_folder_id(selected_folder_id:Number):void {
  this._selected_folder_id = selected_folder_id;
}

/**
 * get tasks by query params.
 * 
 * @param condition
 */
private function _get_tasks_by_quey_params(condition:TaskQueryConditionDto):void {
  //...../tasks/folder/1/statuses/0/classes/1,2/activities/1,2/people/1,2/offset/0/limit/50
  var url:String = "";
  url += Constants.URI_TASKS;
  url += "/folder/${folder_id}";
  url += "/statuses/0";
  url += "/classes/${classes}";
  url += "/activities/${activities}";
  url += "/people/${people}";
  url += "/offset/${offset}";
  url += "/limit/${limit}";

  url = StringUtil.replace(url, "${folder_id}",  new String(condition.folder_id));
  url = StringUtil.replace(url, "${classes}",    condition.classes_to_string());
  url = StringUtil.replace(url, "${activities}", condition.activities_to_string());
  url = StringUtil.replace(url, "${people}",     condition.people_to_string());
  url = StringUtil.replace(url, "${offset}",     new String(this._offset));
  url = StringUtil.replace(url, "${limit}",      new String(this._limit));
  
  var rest:RESTWebService = new RESTWebService();
  rest.do_get(url, _load_tasks_onConnected, _load_onFailure);
}

/**
 * load tasks on connected.
 * 
 * @param e - event.
 */
private function _load_tasks_onConnected(e:Event):void {
  var loader:URLLoader = URLLoader(e.target);
  var response_text:String = loader.data;
  if (response_text == null || response_text == "null" || response_text == ""){
    this.dataProvider = new ArrayCollection(new Array());

  } else {
    var array:Array = (JSON.decode(response_text) as Array);
    var tasks:Array = [];
    
    for each (var t:Object in array) {
      var card_image:Object = 
        Constants.get_task_card_image(
          t.classId, t.statusId, t.personId == Authorize.user_id);
              
      var task:TaskDto = new TaskDto();
      task.id = t.id;
      task.status_id = t.statusId;
      
      task.class_id = t.classId;
      task.class_icon = Constants.get_folder_by_status_and_class(t.statusId, t.classId).icon;
      
      task.activity_id = t.activityId;
      task.activity_color = new Number("0x" + t.activityColor);
      task.person_id = t.personId;
      task.person_firstname = t.personFirstName;
      task.person_lastname = t.personLastName;
      task.title = t.title;
      task.activity_icon =
        "images/default/palette/pl_bg_" + t.activityColor + ".gif";
      task.backgroud_image = card_image.image;
      task.due = t.due;
      tasks.push(task);
    }
    this.dataProvider = new ArrayCollection(tasks);
  }
  this._load_folder_records();
}

/**
 * load folder records.
 * 
 */
private function _load_folder_records():void {
  var url:String = Constants.URI_TASKS + "/action/getFolderRecords";
  var rest:RESTWebService = new RESTWebService();
  rest.do_get(url, _load_rows_of_foler_onConnected, _load_onFailure);
}

/**
 * load on failure.
 * 
 */
private function _load_onFailure(e:Object):void {
  trace('::: error :::');
  trace(e);
}

/**
 * load rows of folder on connected. 
 * 
 * @param e - event
 */
private function _load_rows_of_foler_onConnected(e:Event):void {
  var loader:URLLoader = URLLoader(e.target);
  var response_text:String = loader.data;
  if (response_text == null || response_text == "null" || response_text == ""){
    this.dataProvider = new ArrayCollection(new Array());
    return;
  }
  var array:Array = (JSON.decode("[" + response_text + "]") as Array);
  this.task_folder_dto.inbox_rows = array[0].inbox;
  this.task_folder_dto.tasks_rows = array[0].tasks;
  this.task_folder_dto.today_rows = array[0].today;
  this.task_folder_dto.today_todo_rows = array[0].todo;
  this.task_folder_dto.today_doing_rows = array[0].doing;
  this.task_folder_dto.today_done_rows = array[0].done;
  this.task_folder_dto.week_rows = array[0].week;
  this.task_folder_dto.month_rows = array[0].month;
  this.task_folder_dto.hold_rows = array[0].hold;
  this.task_folder_dto.someday_rows = array[0].someday;
  this.task_folder_dto.archive_rows = array[0].archive;
  this.task_folder_dto.trash_rows = array[0].trash;
}

/**
 * set query condition. 
 *
 * @param condition
 */
private function _set_query_condition(condition:TaskQueryConditionDto):void {
  // classes.
  var classes:Array = [];
  var classes_dataProvider:ArrayCollection =
    this.browse_pane.classes.dataProvider as ArrayCollection;
  
  if (classes_dataProvider == null
   || classes_dataProvider.length == 0) {
   
   classes.push(condition.folder_id);
  } else {
    for each (var c:Object in classes_dataProvider) {
      if (c.selected) classes.push(c.id);
    }
  }
  condition.classes = classes;
  
  // activities.
  var activities:Array = [];
  var activities_dataProvider:ArrayCollection =
    this.browse_pane.activities.dataProvider as ArrayCollection;
  if (activities_dataProvider == null
   || activities_dataProvider.length == 0) {
   
   activities.push(0);
  } else {
    for each (var a:Object in activities_dataProvider) {
      if (a.selected) activities.push(a.id);
    }
  }
  condition.activities = activities;
  
  // person in charges.
  var people:Array = [];
  var people_dataProvider:ArrayCollection =
    this.browse_pane.person_in_charges.dataProvider as ArrayCollection;
  
  if (people_dataProvider == null
   || people_dataProvider.length == 0) {
    
    people.push(Authorize.user_id);
  } else {
    for each (var p:Object in people_dataProvider) {
      if (p.selected) people.push(p.id);
    }
  }
  condition.people = people;
}

/**
 * selected gtd folder. 
 * 
 * @param folder_id
 * @param task screen component
 */
private function _selected_gtd_folder(folder_id:Number):void {
  var classes:Array = Constants.get_classes_for_browser(folder_id);
  var array:Array = [];
  for (var i:Number = 0; i < classes.length; i++) {
    array.push(
      new BrowseClassificationDto(
        classes[i].id,
        classes[i].value,
        classes[i].icon,
        classes[i].selected,
        classes[i].enable)
    );
  }
  this.browse_pane.classes.dataProvider = new ArrayCollection(array);
  this._do_query();
}


/**
 * load activities.
 * 
 */
private function _load_activities():void {
  var rest:RESTWebService = new RESTWebService();
  rest.do_get(
    Constants.URI_ACTIVITIES, _load_activities_onConnected, _load_onFailure);
}

/**
 * load activities on connected.
 * 
 * @param e - event
 */
private function _load_activities_onConnected(e:Event):void {
  var loader:URLLoader = URLLoader(e.target);
  var response_text:String = loader.data;
  
  this._activities = 
    (response_text == null || response_text == "null" || response_text == "")
    ? null
    : JSON.decode(response_text) as Array;
  
  this.browse_pane.setup_activities(this._activities);
  this.new_task_pane.setup_activities(this._activities);
}

/**
 * load person in charge.
 * 
 */
private function _load_person_in_charge():void {
  var rest:RESTWebService = new RESTWebService();
  rest.do_get(
    Constants.URI_ACTIVITY_MEMBERS + "/action/findByBelongActivities",
    _load_person_in_charge_onConnected,
    _load_onFailure
  );
}

/**
 * load person in charge on connected.
 * 
 * @param e - event 
 */
private function _load_person_in_charge_onConnected(e:Event):void {
  var loader:URLLoader = URLLoader(e.target);
  var response_text:String = loader.data;
  var array:Array = 
    response_text == null || response_text == "null" || response_text == ""
    ? null
    :JSON.decode(response_text) as Array;
  this.browse_pane.setup_person_in_charge(array);
}

/**
 * posted new task.
 * 
 */
private function _posted_new_task():void {
  this._load_folder_records();
  this._do_query();
}

/**
 * item on double click.
 * 
 */
private function _item_on_dblclick():void {
  var list_sel:Object = list_view.selectedItem;
  var block_sel:Object = block_view.selectedItem;
  
  if (list_sel == null && block_sel == null) return;
  if (_selected_folder_id == 21) return;

  var selected_id:int = 
    block_view.visible ? block_sel.id : list_sel.id;
  
  var url:String = "";
  url += Constants.URI_TASKS;
  url += "/";
  url += selected_id;
  
  var rest:RESTWebService = new RESTWebService();
  rest.do_get(url, _load_task_by_id_onConnected, _load_onFailure);
}

/**
 * load tasks by id.
 * 
 * @param e - event
 */
private function _load_task_by_id_onConnected(e:Event):void {
  var loader:URLLoader = URLLoader(e.target);
  var response_text:String = loader.data;

  if (response_text == null || response_text == "null" || response_text == ""){
  } else {
    var task:Object = (JSON.decode("[" + response_text + "]") as Array)[0];
    var dto:TaskDto = new TaskDto();
    
    dto.id = task.id;
    dto.activity_color = task.activityColor;
    dto.activity_id = task.activityId;
    dto.activity_name = task.activityName;
    dto.class_id = task.classId;
    dto.status_id = task.statusId;
    dto.title = task.title;
    dto.description = task.description;
    dto.due = task.due;
  
    // show sub window.
    var edit_form:component.TaskEditPopUpWindow =
      component.TaskEditPopUpWindow( 
        PopUpManager.createPopUp(this, component.TaskEditPopUpWindow, true));
    
    edit_form.setTask(dto);
    edit_form.setup_activities(this._activities, dto.activity_id);
    edit_form.setup_folders(Constants.GTD_FOLDERS, dto.class_id, dto.status_id);
    edit_form.after_posted_proc(
      function():void {
        _do_query();
      }
    );
    edit_form.x = (this.width  - edit_form.width)  / 2;
    edit_form.y = (this.height - edit_form.height) / 2;
  }
}

/**
 * empty trash.
 * 
 */
private function _empty_trash():void {
  var url:String = Constants.URI_TASKS + "/action/emptyTrash";
  var rest:RESTWebService = new RESTWebService();
  rest.do_get(
    url,
    function (e:Event):void{
      _do_query();    
    },
    _load_onFailure
  );
}

/**
 * do query.
 * 
 */
private function _do_query():void {
  var condition:TaskQueryConditionDto = new TaskQueryConditionDto();
  condition.folder_id = this._selected_folder_id;
  this._set_query_condition(condition);
  this._get_tasks_by_quey_params(condition);
}

/**
 * move to activity.
 * 
 */
private function _move_to_activity():void {
  this.dispatchEvent(new Event("move_to_activity",false,false));
}
