/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.auth {
  import com.adobe.crypto.SHA1;
  import com.adobe.serialization.json.JSON;
  
  import flash.events.Event;
  import flash.events.IOErrorEvent;
  import flash.net.URLLoader;
  import flash.net.URLLoaderDataFormat;
  import flash.net.URLRequest;
  import flash.net.URLRequestHeader;
  import flash.net.URLRequestMethod;
  import flash.net.URLVariables;
  
  import org.seasar.tugboat.gtd.commons.Constants;
  
  /**
   * authorize. 
   * 
   */ 
  public class Authorize {
    public static var root_url:String;
    
    private static var _url:String;
    private static var _accept:String;
    private static var _user_name:String;
    private static var _password:String;
    private static var _nonce:String;
    private static var _created:String;
    
    private static var _authorize:Authorize;
    private static var _closure:Function;
    
    public static var user_id:int = 0;
    
    private static var MSG_MISS_URL:String     = "Error: Please,Confirm server url.";
    private static var MSG_UNAUTHORIZED:String = "Error: Unauthorized.";
    
    /**
     * get this component.
     * 
     * @param url
     * @param accept
     * @parm closure
     * @return
     */  
    public static function getComponent(
      root_url:String, accept:String, closure:Function):Authorize {

      Authorize.root_url =
        root_url.substring(root_url.length-1) == "/" 
        ? root_url.substring(0,root_url.length-1)
        : root_url;
      
      _url = root_url + Constants.URI_AUTH;
      _accept = accept;
      _closure = closure;
      
      if (_authorize == null) _authorize = new Authorize();
      return _authorize;
    }
    
    /**
     * connecting server.
     * @param user_name
     * @param password
     */
    public function login(user_name:String, password:String):void {
      user_id = 0;
      _user_name = user_name;
      _password = password;
      
      var req:URLRequest = new URLRequest(_url);
      req.method = URLRequestMethod.GET;
      
      var accept:URLRequestHeader = new URLRequestHeader(Constants.HTTP_ACCEPT, _accept);
      req.requestHeaders.push(accept);
      req.contentType = _accept;
  
      var loader:URLLoader = new URLLoader();
      loader.dataFormat = URLLoaderDataFormat.TEXT;
      loader.addEventListener(Event.COMPLETE, onConnected);
      loader.addEventListener(IOErrorEvent.IO_ERROR, onLogionFailure);
      loader.load(req);
    }
      
    /**
     * authorize.
     * @param e
     */
    private function onConnected(e:Event):void {
      // get nonce and created.
      var wsse_token_loader:URLLoader = URLLoader(e.target);
      var response_text:String = wsse_token_loader.data;
      if (response_text == null 
       || response_text == ""
       || response_text.indexOf("<!DOCTYPE html") > -1) {

        _closure(false, MSG_MISS_URL);
        return;
      }
      
      var arr:Array = (JSON.decode("[" + response_text + "]") as Array);
      _nonce = arr[0].nonce;
      _created = arr[0].createdAt;
      
      // login processing.
      var req:URLRequest = new URLRequest(_url);
      var accept:URLRequestHeader =
        new URLRequestHeader(Constants.HTTP_ACCEPT, _accept);
      req.requestHeaders.push(accept);
      req.contentType = _accept;
      
      var wsse:URLRequestHeader =
        new URLRequestHeader(Constants.HTTP_WSSE, getWSSEUsernameToken());
      req.requestHeaders.push(wsse);
  
      var url_vars:URLVariables = new URLVariables();
      url_vars.userName = _user_name;
      url_vars.passwordDigest = getPasswrdDigest();
      url_vars.nonce = _nonce;
      url_vars.created = _created;
      req.data = url_vars;
  
      var loader:URLLoader = new URLLoader();
      loader.dataFormat = URLLoaderDataFormat.TEXT;
      loader.addEventListener(Event.COMPLETE, onLoginSuccessful);
      loader.addEventListener(IOErrorEvent.IO_ERROR, onLogionFailure);
      loader.load(req);
    }
    
    /**
     * on login successful.
     * @param e
     */ 
    private function onLoginSuccessful(e:Event):void {
      var user_id_loader:URLLoader = URLLoader(e.target);
      var response_text:String = user_id_loader.data;
      
      if (response_text == "Unauthorized") {
        _closure(false, MSG_UNAUTHORIZED);
        
      } else {
        //_message.text = "";
        user_id = new int(response_text);
        _closure(true,"");
      }
    }
    
    /**
     * on login failure.
     * @param e
     */ 
    private function onLogionFailure(e:Event):void {
      _closure(false, MSG_MISS_URL);
    }
    
    /**
     * get password digest.
     * @return password digest.
     */ 
    private static function getPasswrdDigest():String {
      return SHA1.hashToBase64(_nonce + _created + _password);
    }
    
    /**
     * get wsse string.
     * @return wsse.
     */
    public static function getWSSEUsernameToken():String {
      var password_digest:String = getPasswrdDigest();
      var token:String = new String("UsernameToken ");
      token += "Username=\""       + _user_name       + "\", " 
            +  "PasswordDigest=\"" + password_digest  + "\", " 
            +  "Nonce=\""          + _nonce           + "\", " 
            +  "Created=\""        + _created         + "\"";
      return token;
    }
  }
}