/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.dao{
  import com.adobe.utils.StringUtil;
  
  import flash.data.SQLConnection;
  import flash.data.SQLResult;
  import flash.data.SQLStatement;
  import flash.events.SQLErrorEvent;
  import flash.events.SQLEvent;
  import flash.filesystem.File;
  import flash.filesystem.FileMode;
  import flash.filesystem.FileStream;
  
  import org.seasar.tugboat.gtd.dto.ServerURLDto;
      
  public class ServerURLDao {
    private var _conn:SQLConnection;
    private var _stmt:SQLStatement;
    private var _closure:Function;
    private var _is_selected_data:Boolean = false;
    
    /**
     * @param closure
     */
    public function ServerURLDao(closure:Function) {
      trace('ServerURLDao.init');
      this._closure = closure;
      
      this._open_database();
      this.select();
    }
    
    /**
     * open database. 
     * 
     */
    private function _open_database():void {
      var database_file:File = File.applicationDirectory;
      trace(database_file.nativePath);
      
      database_file = database_file.resolvePath("tugboat-gtd-air.db");
      _conn = new SQLConnection();
      _conn.addEventListener(SQLEvent.OPEN, onDatabaseOpen);
      _conn.addEventListener(SQLErrorEvent.ERROR, onDatabaseError);
      _conn.open(database_file);
    }
    
    /**
     * is connected.
     * @return 
     */
    public function is_selected_data():Boolean {
      return this._is_selected_data;
    }
    
    /**
     * on database open.
     * @param event
     */ 
    private function onDatabaseOpen(event:SQLEvent):void {
      trace("#successful!");
      this._is_selected_data = true;
    }
    /**
     * on database error.
     * @param event 
     */    
    private function onDatabaseError(event:SQLErrorEvent):void {
      trace("#error!");
      this._is_selected_data = false;
    }
    
    /**
     * create table. 
     */
    public function create_table():void {
      var sql:String = this._get_sql("ServerURLDao_createTable.sql");
      execute(sql,
        function():void{
          insert();
        }, null
      );
    }
    
    /**
     * insert a record. 
     */
    public function insert():void {
      var sql:String = this._get_sql("ServerURLDao_insert.sql");
      
      execute(sql, function():void{
          select();
        }, null
      );
    }
    
    /**
     * update
     * @param server_url 
     * 
     */
    public function update(server_url:ServerURLDto):void {
      var sql:String = this._get_sql("ServerURLDao_update.sql");
      sql = StringUtil.replace(sql, "${url}", server_url.url);

      execute(sql, function():void{
        }, null
      );
    }
    
    /**
     * select records. 
     */
    public function select():void {
      trace("::: select :::");
      var sql:String = "SELECT * FROM server_url"; //this._get_sql("ServerURLDao_select.sql");
      this.execute(sql, select_onResult, select_onError);
    }
    
    /**
     * execute sql. 
     * @param sql
     * @param closure on success - function
     * @param closure on error   - function
     */ 
    private function execute(
      sql:String, closure_onSuccess:Function, closure_onError:Function):void {
      
      try {
        _stmt = new SQLStatement();
        _stmt.sqlConnection = _conn;
        _stmt.addEventListener(SQLEvent.RESULT, closure_onSuccess);
        _stmt.addEventListener(
          SQLErrorEvent.ERROR,
          closure_onError == null ? onDatabaseError : closure_onError
        );
        _stmt.text = sql;
        _stmt.execute();

      } catch (e:Error) {
        trace(e);
        this._closure("", false);
      }
    }
    
    /**
     * select on result. 
     * @param event
     */
    private function select_onResult(event:SQLEvent):void {
      var result:SQLResult = _stmt.getResult();
      for each (var columns:Object in result.data) {
        this._closure(columns["url"], true);
        break;
      }
    }
    
    /**
     * select on error.
     * @param event
     */
    private function select_onError(event:SQLErrorEvent):void {
      this.create_table();
    }
    
    /**
     * get sql. 
     * @param file_name
     * @return sql
     */
    private function _get_sql(file_name:String):String {
      var sql_file:File =
        File.applicationDirectory.resolvePath("sql/" + file_name);

      var file_stream:FileStream = new FileStream();
      file_stream.open(sql_file, FileMode.READ);

      var sql:String = file_stream.readUTFBytes(sql_file.size);
      return sql;
    }

  }
}