/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.webservices{
  import flash.events.Event;
  import flash.events.IOErrorEvent;
  import flash.events.SecurityErrorEvent;
  import flash.net.URLLoader;
  import flash.net.URLLoaderDataFormat;
  import flash.net.URLRequest;
  import flash.net.URLRequestHeader;
  import flash.net.URLRequestMethod;
  import flash.net.URLVariables;
  
  import org.seasar.tugboat.gtd.auth.Authorize;
  import org.seasar.tugboat.gtd.commons.Constants;

  public class RESTWebService {
    
    /**
     * @param wsse 
     * 
     */
    public function RESTWebService():void {}
    
    /**
     * do get request. 
     * 
     * @param url
     * @param closure_successful - Function.
     * @param closure_error      - Function.
     */
    public function do_get(
      url:String, closure_successful:Function, closure_error:Function):void {

      trace(url);

      var request:URLRequest = new URLRequest(Authorize.root_url + url);
      request.method = URLRequestMethod.GET;

      var wsse:URLRequestHeader =
        new URLRequestHeader(Constants.HTTP_WSSE, Authorize.getWSSEUsernameToken());
      request.requestHeaders.push(wsse);
      
      var accept:URLRequestHeader =
        new URLRequestHeader(Constants.HTTP_ACCEPT, Constants.HTTP_ACCEPT_JSON);
      request.requestHeaders.push(accept);
      request.contentType = Constants.HTTP_ACCEPT_JSON;
  
      var loader:URLLoader = new URLLoader();
      loader.dataFormat = URLLoaderDataFormat.TEXT;
      loader.addEventListener(Event.COMPLETE, closure_successful);
      loader.addEventListener(IOErrorEvent.IO_ERROR, closure_error);
      loader.addEventListener(IOErrorEvent.NETWORK_ERROR, closure_error);
      loader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, closure_error);
      
      loader.load(request);
    }
    
    /**
     * service [post, update, delete]
     * 
     * @param http method
     * @param url
     * @param variables
     * @param successful function
     * @param error function
     * 
     */
    private function _service(
      http_method:String, url:String, variables:URLVariables, 
      closure_successful:Function, closure_error:Function):void {

      trace(url);

      var request:URLRequest = new URLRequest();
      request.url = url;
      request.method = URLRequestMethod.POST;
      request.data = variables;
      trace(variables);
      
      if (http_method != Constants.HTTP_POST) {
        var http_method_override:URLRequestHeader = 
          new URLRequestHeader(Constants.HTTP_METHOD_OVERRIDE, http_method);
        request.requestHeaders.push(http_method_override);
      }
      var wsse:URLRequestHeader =
        new URLRequestHeader(Constants.HTTP_WSSE, Authorize.getWSSEUsernameToken());
      request.requestHeaders.push(wsse);
      
      var loader:URLLoader = new URLLoader();
      loader.dataFormat = URLLoaderDataFormat.TEXT;
      
      loader.addEventListener(Event.COMPLETE, closure_successful);
      loader.addEventListener(IOErrorEvent.IO_ERROR, closure_error);
      loader.addEventListener(IOErrorEvent.NETWORK_ERROR, closure_error);
      loader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, closure_error);
      
      loader.load(request);
    }
    
    /**
     * do post request. 
     * 
     * @param url
     * @param http_method
     * @param variables
     * @param successful function
     * @param error function
     */
    public function do_post(
      url:String, variables:URLVariables, closure_successful:Function, closure_error:Function):void {
      
      this._service(Constants.HTTP_POST, url, variables, closure_successful, closure_error);
    }
    
    /**
     * do update.
     * 
     * @param url
     * @param variables
     * @param successful function
     * @param error function
     */
    public function do_update(
      url:String, variables:URLVariables, closure_successful:Function, closure_error:Function):void {
        
      this._service(Constants.HTTP_UPDATE, url, variables, closure_successful, closure_error);
    }
    
    /**
     * do delete.
     * 
     * @param url
     * @param variables
     * @param successful function
     * @param error function
     */
    public function do_delete(
      url:String, variables:URLVariables, closure_successful:Function, closure_error:Function):void {
        
      this._service(Constants.HTTP_DELETE, url, variables, closure_successful, closure_error);
    }
  }
}